(function ($, ko, ns) {
    ns.ConfigNodeModel = function (node, parent, name, options) {
        if (!options) {
            options = {};
        }
		this.nodeName = ko.observable(name || "New Node");
		this.identityToken = { nodeName: this.nodeName }; // used for comparison, A == A only if they are one and same instance
		this.parent = parent || null; // ConfigNodeViewModel
		this.properties = ko.observableArray(); // map - key: name; value: {value, type}
		this.children = ko.observableArray(); // [ConfigNodeViewModel]
		this.childrenTokens = ko.observableArray();
		this.configNodeType = options.type || ns.config.nodeTypes.node;
		this._construct(node, options.isDirty);
	};
	
	ns.ConfigNodeModel.prototype = function () {
		var	
			addChild = function (nodeModel) {
				this.children.push(nodeModel);
				this.childrenTokens.push(nodeModel.identityToken);
			},
			
			removeChild = function (childModel) {
				this.children.remove(childModel);
				this.childrenTokens.remove(childModel.identityToken);
			},
			
			remove = function (childModel) {
				this.parent.removeChild(this);
			},
			
			// serializes in JSON format, but the result is not a valid JSON string
			serialize = function () {
			    var 
					json = "",
					i = 0,
					props = this.properties(),
					prop = null,
					children = this.children();

			    json += ko.toJSON(this.nodeName());

			    if (this.configNodeType === ns.config.nodeTypes.node) {
                    json += ":{";
			    } else if (this.configNodeType === ns.config.nodeTypes.mapProperty) {
                    json += ":[{";
			    } else if (this.configNodeType === ns.config.nodeTypes.arrayProperty) {
                    json += ":[[";
                }
				
				for (i = 0; i < props.length; i++) {
					prop = props[i];
					json += ko.toJSON(prop.key) + ':[' + ko.toJSON(prop.value) + ',' + ko.toJSON(prop.type) +'],';
				}
				
				for (i = 0; i < children.length; i++) {
					json += children[i].serialize();
					json += ",";
				}

				if (json.lastIndexOf(",") === json.length - 1) {
					json = json.substring(0, json.length - 1);
				}
				
				if (this.configNodeType === ns.config.nodeTypes.node) {
				    json += "}";
				} else if (this.configNodeType === ns.config.nodeTypes.mapProperty) {
				    json += '}, "' + ns.config.nodeTypes.mapProperty + '"]';
				} else if (this.configNodeType === ns.config.nodeTypes.arrayProperty) {
				    json += '], "' + ns.config.nodeTypes.arrayProperty + '"]';
				}

				return json;
			},
			
			toJSON = function () {
				var
					json = "";
				
				json += this.serialize();
				
				if (this.parent !== null) {
					json = "{" + json + "}";
				} else {
					json = json.substring(json.indexOf(":") + 1);
				}
				
				return json;
			},
			
			_getConfigNodeProperty = function (propertyName, propertyValue) {
				var 
					property = {};
				
				property.key = ko.observable(propertyName);
				property.type = ko.observable(propertyValue[1]);
				property.value = ko.observable(propertyValue[0]);

				return property;
			},

			_construct = function (node, isDirty) {
				var 
					prop = null,
					currentValue = null,
					properties = this.properties(),
					children = this.children();
				
				if (node) {
					if (typeof this.parent !== "undefined" && this.parent !== null) {
						this.parent.childrenTokens.push(this.identityToken);
					}

					for (prop in node) {
						currentValue = node[prop];
						if (typeof currentValue !== "object") {
						    continue;
						}
						
						if ($.isArray(currentValue)) {
						    if (!ns.ConfigNodeModel.isConfigNodeType(currentValue[1])) {
						        properties.push(_getConfigNodeProperty(prop, currentValue));
						    } else if (currentValue[1] === ns.config.nodeTypes.mapProperty) {
						        children.push(new ns.ConfigNodeModel(currentValue[0], this, prop, { type: ns.config.nodeTypes.mapProperty }));
						    } else if (currentValue[1] === ns.config.nodeTypes.arrayProperty) {
						        children.push(new ns.ConfigNodeModel(currentValue[0], this, prop, { type: ns.config.nodeTypes.arrayProperty }));
						    }
						} else {
						    children.push(new ns.ConfigNodeModel(currentValue, this, prop, { type: ns.config.nodeTypes.node }));
						} 
					}

					children.sort(ns.utils.configNodeComparator);

					this.properties.valueHasMutated();
					this.children.valueHasMutated();				
				} else {
					if (typeof this.parent !== "undefined" && this.parent !== null) {
						this.parent.addChild(this);
					}
				}
				
				this.dirtyFlag = new ko.DirtyFlag([this.nodeName, this.properties, this.childrenTokens], !!isDirty);
				this.isDirty = ko.computed(function () {
				    return this.dirtyFlag().isDirty();
				}, this);
			};
			
		return {
			addChild: addChild,
			remove: remove,
			removeChild: removeChild,
			serialize: serialize,
			toJSON: toJSON,
			_construct: _construct,
			type: "ConfigNode"
		}
	}();
	
	ns.ConfigNodeModel.isConfigNodeType = function (typeToCheck) {
	    var type = null,
            types = ns.config.nodeTypes;
	    for (type in types) {
	        if (types[type] === typeToCheck) {
	            return true;
	        }
	    }
	    return false;
	};
}($, ko, window.configTree = window.configTree || {}));