#! /bin/sh
#########################################################################################
# Update from a directory (on a USB key for example).
# 
# 2014 Logilin for FDE
# 20/10/2014 : Modified by FDE <a.courrier@fde.fr>
# 14/11/2014 : Modified by HAGER <m.courivaud@hager.fr>
#
# This script is included in a zipfile that have been previously handled by an update 
# daemon which had opened it, verified that a so called "update.sh" script was included
# before lanching it. That's how this script is started.
#
# Versions
#
# 1.0.3
# Add support of a $MODENV_DIR directory that may contain (could be empty or not exists)
# files and scripts that have to be executed to adapt operating system environment for
# full new functionalities completion. If detected, the script $MODENV_SCRIPT is
# launched. To be fully compatible with a backup functionality, the $MODENV_SCRIPT must
# have 2 entry points, one for installing new environment, another to backup previously
# running environment
# 
# Add logs from mbsa_stop to log-file, so that we can verify how the framework was stopped
#
# Make the power led blinking green until the end of the process where the application 
# launching script is started. That is its responsability to manage the final power led 
# color according to the application status
# 
# 1.0.4
# Add support of both firmware and software update in the same session if the firmware 
# file is included in the zipfile
#
# 1.0.5
# Modify SetLed function to support more colors and modes
# 
# 1.0.6
# The script gets its own version and don't share the SCRIPT_VERSION variable anymore
#
# 1.0.7
# Reorganize update_firmware and update_software functions 
# Make some adjustment so that each function : udate software, update firmware and
# update environment could be used separetly or combined
# 
# 1.0.8
# Move ReplaceUpdateDaemon function to modify_environment script which is its natural
# location. Moved also KillRunningProcess and IsProcessRunning associated functions
#
# 1.0.9
# Set the $MODENV_DIR to absolute instead of relative path. It's useful to prevent errors
# due to a directory change made in a function which forgot to goes back to $UNZIP_DIR 
# directory.
#
# Correct the Update_Firmware function which was changing directory at the beginning but
# forgot to re-establish it to $UNZIP_DIR in case of error.
# 
# Change some message during main process execution to a better view of the location  
# after the differents sub-process execution.
#
# 1.1.0
# Change some signalization to locate precisely which sub-process is currently working
# and signify global process ending state
# 
# 1.2.3
# Change some message during restore process to be more explicit
# Correction of mispelled variable in StartRestore function
# 
# 1.2.4
# Tiny modifications on log message to be more readable
# Add a small delay to error signalization on missing starting software script at the end 
# of the update process.
#########################################################################################

UPDATE_SCRIPT_VERSION="1.2.4"
UPDATE_SCRIPT_INITIALS="[U--]"

source /usr/local/bin/update_common
source /usr/local/bin/BSPUpdateTools

#########################################################################################
# Local variables defined
#########################################################################################

ROOT_DIR=/root

APP_DIR=$ROOT_DIR/app
BACKUP_APP_DIR=$ROOT_DIR/oldapp

MBSA_DIRECTORY=mbsa
OSGI_DIRECTORY=osgi

MBSA_RUNNING="mbsae.core"
JAVA_RUNTIME="java"

INIT_DIR="/etc/init.d"
WATCHDOG_SCRIPT="S15watchdog"

USR_LOCAL_BIN="/usr/local/bin"
HAGER_APP_STARTING_SCRIPT="application"

KNX_MBSA_START="knxng_mbsa_start"
EMSS_MBSA_START="emss_mbsa_start"
APP_MBSA_START=""

FUNC_INSTALL="Install"
FUNC_RESTORE="Restore"
FUNC_CLEAN="Clean"

UPDATE_DIR="/tmp/update"
APP_STATUS_FILE="$UPDATE_DIR/platformstate/started"
UPDATE_STATUS_FILE="$UPDATE_DIR/status"

DIR_USR="usr"
DIR_USR_LOCAL="$DIR_USR/local"
DIR_USR_LOCAL_BIN="$DIR_USR_LOCAL/bin"

APP_PROCESS_TO_CATCH="mbsae.core"

# ------------ Firmware defines

FW_PREFIX=HBoxFirmware
FW_VERSION_FILE="/proc/HBox-Firmware-version"
DEFAULT_FW_VERSION=0.0.0

# ------------ Zipfile structure

MODENV_DIR="modify_environment"
MODENV_SCRIPT="modify_environment.sh"
COMMON_LIBRARY="update_common"

#------------- Various

DD_CMD="dd if"

#----------------------------------------------------------------------------------------
# 										TAKE CARE 
# The variables defined below exist also in the "update_common" file
# Therefore their values must be similar to maintain the whole process logic
#----------------------------------------------------------------------------------------

#------------------ Status defines 

STATUS_UPDATE_OK=0
STATUS_SURVEY=1
STATUS_UPDATE_ABORT=2
STATUS_UPDATE_FILE_ERROR=3
STATUS_WAIT_APP=4
STATUS_APP_STARTED=5
STATUS_UPDATE_FILE_OK=6

#------------------ LogFiles defines

DATA_VAR_DIR=/data/var
DATA_VAR_LOG_DIR=$DATA_VAR_DIR/log
APP_LOGFILE=$DATA_VAR_LOG_DIR/update

LOGFILE_AND_CONSOLE="Console"

#########################################################################################
# Functions
#########################################################################################

#########################################################################################
# Function that create a directory with a given name. Verify first that a file with the 
# same name doesn't exist at the designed location because it will prevent directory 
# creation. 
#
# Input - $1 Directory name
#
# IMPORTANT
# The "CreateDirectory" function also exists in "update_common" file but not since the 
# first version. So in case "update_common" can't be sourced (which is done at the very 
# beginning of the current script), this function will be used to log the error.

Update_CreateDirectory() {
	
	if [ ! -z $1 ]; then 																											# The directory name must be given at the least 

		if [ ! -e "$1" ]; then																										# If directory already exists, just do nothing

			if [ -f "$1" ]; then 																									# Verify if a file with the same name already exists at current location
				rm "$1"																												# If so, it must be deleted, otherwise it will prevent the directory to be created
			fi

			mkdir "$1"
		fi
	fi
}

#########################################################################################
# Function that log a message in logfile
#
# Input - $1 Message
# 		  $2 Severity level of the message : NORMAL - WARNING
# 		  $3 Destination : LOGFILE - LOGFILE + CONSOLE
#
# IMPORTANT
# The "WriteLog" function also exists in "update_common" file but not since the 
# first version. So in case "update_common" can't be sourced (which is done at the very 
# beginning of the current script), this function will be used to log the error.

Update_WriteLog() {

	# Create the log file and directories if needed

	if [ ! -e "$DATA_VAR_DIR" ]; then
		Update_CreateDirectory $DATA_VAR_DIR
	fi

	if [ ! -e "$DATA_VAR_LOG_DIR" ]; then
		Update_CreateDirectory $DATA_VAR_LOG_DIR
	fi

	LogDate=`date +%Y/%m/%d-%H:%M:%S`

	# The second parameter (optional) manage the message severity level
	# If not defined, the message is always written

	if [ -z $2 ] || [ $2 -ge $logLevel ]; then
		echo "$LogDate $UPDATE_SCRIPT_INITIALS : $1" >> $APP_LOGFILE
	fi

	# The third parameter (optional) allows the message to be displayed on the console

	if [ ! -z $3 ] && [ "$3" == "$LOGFILE_AND_CONSOLE" ]; then
		TimeElapsed=`awk '{printf("[%11f]", $1)}' /proc/uptime)`
		echo "$TimeElapsed $1" >> $CONSOLE
	fi
}

#########################################################################################
# Function that set correct defines according to the current board type
Update_Init() {

	# Init specific hardware defines
	boardtype=$(Get_BoardType)
 	LogUpdate "$boardtype board detected"

 	case "$boardtype" in

 		$KNX_BOARD)
			APP_MBSA_START=$KNX_MBSA_START
			;;

		$EMSS_BOARD)
			APP_MBSA_START=$EMSS_MBSA_START
			;;

		*)
			return 1
			;;

	esac

	return 0
}

#########################################################################################
# Function for setting internal status that will be used by all update sequence logic
#
# Input : $1 Status to set 
#
# IMPORTANT
# This function already exists in "update_common" file but not since the first version
# So in case this file can't be actualized (sourced) at the very beginning of the script, 
# this function will be used to log the error.

InternalSetUpdateStatus() {
	if [ ! -z "$1" ]; then
		echo "$1" > $UPDATE_STATUS_FILE
	fi
}

###########################################################################################
# Function that add the initials of the script to each message that will be logged.
# That makes the reading of the logs unambiguous

WriteLog() {
	LogUpdate "$UPDATE_SCRIPT_INITIALS : $1" $2 $3
}

###########################################################################################
# Function that restart the watchdog daemon.
#
# Used before killing hardly the framework.
# While a normal ending will automatically restore it, a hard stop doesn't and it must be done
# manually otherwise the box will reboot after 20 seconds.

Restart_WatchDog_Daemon() {

	if [ -e "$INIT_DIR/$WATCHDOG_SCRIPT" ]; then
		LogUpdate "$INIT_DIR/$WATCHDOG_SCRIPT script launched with start parameter" $LOG_DEBUG
		/etc/init.d/S15watchdog start 2>&1 >> $APP_LOGFILE
	else
		LogUpdate "WARNING -- $INIT_DIR/$WATCHDOG_SCRIPT script not found. Daemon not restarted"
	fi
}

#########################################################################################
Is_Framework_Running() {
	
	if IsProcessRunning $MBSA_RUNNING; then
		LogUpdate "$MBSA_RUNNING still running" $LOG_DEBUG
		return 0
	fi

	if IsProcessRunning $JAVA_RUNTIME; then
		LogUpdate "$JAVA_RUNTIME still running" $LOG_DEBUG
		return 0
	fi

	LogUpdate "Neither $MBSA_RUNNING nor $JAVA_RUNTIME are currently running" $LOG_DEBUG
	return 1
}

#########################################################################################
Get_BSP_Version() {

	if [ ! -e $BSPVERSION_FILE ]; then
		return 1
	fi

	bspversion=$(cat "$BSPVERSION_FILE")
	return 0
}

#########################################################################################
IsSoftwarePresent() {

	if [ -d $UPDATE_APP_DIR ]; then
		LogUpdate "Software directory detected. Starting analysis"
		return 0
	else
		return 1
	fi
}

#########################################################################################
# Function that compare 2 numbers given in entry in the form of : X.Y.Z
#
# Return
# 	- 1 : $2 > $1
#	- 0 : All other cases

Is_Newer_Firmware_Version() {

	if [ -z "$1" ] || [ -z "$2" ]; then
		LogUpdate "No version to compare. Abort"
		return 0
	fi

	LogUpdate "Compare current version = $1 and new version = $2" $LOG_DEBUG

	# Extract major, minor and release versions from current firmware

	Current_Major=`echo $1 | awk -F. '{print $1}'`
	Current_Minor=`echo $1 | awk -F. '{print $2}'`
	Current_Release=`echo $1 | awk -F. '{print $3}'`

	# Extract major, minor and release versions from new firmware

	New_Major=`echo $2 | awk -F. '{print $1}'`
	New_Minor=`echo $2 | awk -F. '{print $2}'`
	New_Release=`echo $2 | awk -F. '{print $3}'`

	# Build combined version numbers

	Current=$(((Current_Major*100)+(Current_Minor*10)+Current_Release))
	New=$(((New_Major*100)+(New_Minor*10)+New_Release))

	# And compare them

	if [ $New -gt $Current ]; then
		return 0
	else
		return 1
	fi

}

#########################################################################################
# Function that compare the version of a firmware whose name is given in entry to the 
# current one
#
# 	$1 : firmware filename

Is_Firmware_Usable() {

	if [ -z "$1" ]; then
		LogUpdate "Is_Firmware_Usable : Empty argument, valid firmware filemane expected"
	fi

	newFirmware=$1 																												# Keep whole firmware filename
	returnCode=1

	NewFW_Version=`echo $newFirmware | awk -F- '{print $2}'`																	# Extract version numbers
	LogUpdate "New firmware version : $NewFW_Version"

	##### Get current firmware version

	if [ -f "${FW_VERSION_FILE}" ] ; then

		CurrentFW_Version=`cat "$FW_VERSION_FILE" | grep '^[0-9]\+\.[0-9]\+\.[0-9]\+$'`

		if [ "${CurrentFW_Version}" = "" ] ; then
	      	LogUpdate "Current firmware version is unknown : use default version"
			CurrentFW_Version=${DEFAULT_FW_VERSION}
		fi

	else
      	LogUpdate "$FW_VERSION_FILE not found. Use default version"
		CurrentFW_Version=${DEFAULT_FW_VERSION}
	fi

	LogUpdate "Current firmware version : $CurrentFW_Version"

	##### Only upgrade are allowed, never downgrade, so compare the versions
	if Is_Newer_Firmware_Version $CurrentFW_Version $NewFW_Version; then
		
		LogUpdate "Found a newer firmware version: $NewFW_Version. Verifying its validity"

		##### Destroy remaining firmware file if exists
		if [ -f /tmp/rootfs.ext2 ] ; then
			rm -f /tmp/rootfs.ext2																										
		fi

		##### Now compare the checksum contained in header to those computed
		read_sum=`$DD_CMD="$newFirmware" bs=1 count=32`																					# Read checksum from header

		##### Extract firmware
		LogUpdate "$newFirmware : extracting file"
		dd if="$newFirmware" of="/tmp/rootfs.ext2" bs=4k skip=1 																		# Those checksums are contained in an otherwise empty 4K block.

		##### Compute checksum of the extracted file
		comp_sum=`md5sum < "/tmp/rootfs.ext2" | dd bs=1 count=32`

		##### And compare the two sums.
		if [ "$comp_sum" = "$read_sum" ]; then
			LogUpdate "$newFirmware verified. START UPDATE ($CurrentFW_Version --> $NewFW_Version)"
		   	returnCode=0 																												# Return success
		else
		   	LogUpdate "$newFirmware WRONG CRC. Abort"
		fi

	else
		LogUpdate "Firmware file version is not newer than current one. Abort update firmware process"
	fi

	return $returnCode
}

#########################################################################################
Stop_Software() {

	# Activate Prosyst script to stop the framework and therefore the software
	if [ -f "$APP_DIR/$MBSA_DIRECTORY/bin/mbsa_stop" ]; then
		LogUpdate "Stopping framework"
		"$APP_DIR"/"$MBSA_DIRECTORY"/bin/mbsa_stop 2>&1 >> $APP_LOGFILE
	else
		LogUpdate "$APP_DIR/$MBSA_DIRECTORY/bin/mbsa_stop not found"
	fi

	# Verify the job went well before upgrading the application
	if Is_Framework_Running; then

		# Hum, not so well, try a different approach
		LogUpdate "Framework still running. Stopping it the hard way"

		# We'll be more hard now but first start manually watchdog daemon to avoid platform reboot
		LogUpdate "Force watchdog daemon's restart"
		Restart_WatchDog_Daemon

		# And try to kill individual processes
		if ! KillRunningProcess $MBSA_RUNNING; then
			LogUpdate "ERROR : cannot stop $MBSA_RUNNING. Framework still alive. Update process abort"
			return 1
		fi


		if ! KillRunningProcess $JAVA_RUNTIME; then
			LogUpdate "ERROR : cannot stop $JAVA_RUNTIME. Framework still alive. Update process abort"
			return 1
		fi

	fi

	LogUpdate "Framework stopped"
}

#########################################################################################
Update_Software() {
	
	#------------------ Stop the framework before moving or deleting files
	if Is_Framework_Running; then

		LogUpdate "Framework is currently running. Beginning stopping process"

		# Before stopping the software, a socket must be opened on localhost, port 23174 that 
		# will create a backup of the software data. The socket will be automatically closed 
		# once the backup is done.

		LogUpdate "Notify application that it will be stopped by opening a socket on localhost - port 23174"
		ssh 127.0.0.1 -p 23174 2>&1 >> $APP_LOGFILE

		# Remove application status file as it had just been created by the application 

		if [ -f $APP_STATUS_FILE ]; then
			LogUpdate "Remove application status file : $APP_STATUS_FILE"
			rm "$APP_STATUS_FILE" 2>&1 >> $APP_LOGFILE
		else
			LogUpdate "No application status file : $APP_STATUS_FILE to remove"
		fi

		# Then stop the framework
		if ! Stop_Software; then
			return 1
		fi

	fi

	# Then, clean potential existing old backup directory 
	if [ -f "$BACKUP_APP_DIR" ]; then
		LogUpdate "Old backup directory found. Suppressed"
		CleanDirectory $BACKUP_APP_DIR
	fi

	# Make a backup of the current version that will be used in case of update failure
	LogUpdate "Backup current application in $BACKUP_APP_DIR"
	mv "$APP_DIR" "$BACKUP_APP_DIR" 2>&1 >> $APP_LOGFILE

	# Then create a new app directory as the old one just disappeared
	LogUpdate "Make a new application directory : $APP_DIR"
	CreateDirectory $APP_DIR 2>&1 >> $APP_LOGFILE

	# And finally copy new application files to correct location
	LogUpdate "Install new application files from $UPDATE_APP_DIR to $APP_DIR "
	cp -R "$UPDATE_APP_DIR"/* "$APP_DIR" 2>&1 >> $APP_LOGFILE

	# Then verify that there are some files copied under the $UPDATE_APP_DIR
	directory_content=$(ls -a $UPDATE_APP_DIR | sed -e "/\.$/d" | wc -l)
	
	if [ $directory_content -eq 0 ]; then
		LogUpdate "Something goes wrong as there is not any file in $UPDATE_APP_DIR"
		return 1
	fi

	# Set executable permission on scripts copied as the permission are not preserved during the copy process
	LogUpdate "Set executable scripts"

	chmod 755 "$APP_DIR"/"$MBSA_DIRECTORY"/bin/mbsa_start 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$MBSA_DIRECTORY"/bin/mbsa_stop 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$MBSA_DIRECTORY"/bin/$APP_MBSA_START 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$MBSA_DIRECTORY"/lib/runtimes/linux/Hager-ARMv7/mbsae.core 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$OSGI_DIRECTORY"/bin/vms/jdk/server 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$OSGI_DIRECTORY"/bin/vms/jdk/profiler 2>&1 >> $APP_LOGFILE
	chmod 755 "$APP_DIR"/"$OSGI_DIRECTORY"/bin/vms/*.sh 2>&1 >> $APP_LOGFILE
	
	return 0
}

#########################################################################################
# Function that update a firmware.
#
# First verify if a firmware is present then verify its validity exactly the same way it 
# is verified by the update_from_directory script which is in charge of updating the firmware
# by USB, then finally and like the update_from_directory script, launch the install_firmware 
# script which will do the job

Update_Firmware() {

	#### Save current directory and switch on firmware specific directory
	currentDir=$(pwd)
	cd "$UPDATE_FIRM_DIR"

	LogUpdate "Looking at the content of $(pwd) directory"
	
	##### Then look at the directory content to try to find a firmware file
	for fw in `ls -r .`
	do

		##### We must consider the case of an empty directory which is not an error at all
		if [ "$fw" = "*" ] ; then
			LogUpdate "$UPDATE_FIRM_DIR : No file found at all"
			cd $currentDir
			return 0
		fi
		
		##### And also the case of the presence of files which are not firmware files
		##### And once again, it's not an error and must not be considered as such
		if echo "$fw" | grep -i "^${FW_PREFIX}-[0-9]\+\.[0-9]\+\.[0-9]\+"'$' > /dev/null										# Looking for a specific file pattern / ^ ignore case for prefix! :-)
		then
			newFirmware="$fw"
		else
			LogUpdate "$UPDATE_FIRM_DIR : No valid firmware file found"
			cd $currentDir
			return 0
		fi

	done	

	##### A file that looks like a firmware has been found, then compare its version with the 
	##### current one, as we just consider to do upgrade neither downgrade, and verify its checksum

	if Is_Firmware_Usable "$newFirmware"; then

		SetLed POWER GREEN HEARTBEAT

		if [ -f /tmp/BSP_UPDATE_OK ] ; then
			rm /tmp/BSP_UPDATE_OK
		fi

		##### Start update script
		/usr/local/bin/install_firmware.sh

	fi

	cd $currentDir
}

#########################################################################################
# Function that source the new common lib which must be provided with the update zipfile

Update_Source_New_Common_Lib() {

	Update_WriteLog "Common library version actually running : $UPDATE_COMMON_SCRIPT_VERSION"

	# Look at new common library's presence

	if [ ! -d "$UPDATE_ENV_DIR" ]; then
		Update_WriteLog "ERROR ----- $UPDATE_ENV_DIR must exists but it doesn't !!"
		return 1
	fi

	if [ ! -f "$UPDATE_ENV_DIR/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY" ]; then
		Update_WriteLog "ERROR ----- $UPDATE_ENV_DIR/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY can't be found !!"
		return 1
	fi

	# Everything fine, just source the new common library so that we can use its functionalities

	Update_WriteLog "Sourcing $UPDATE_ENV_DIR/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY"
	source $UPDATE_ENV_DIR/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY

	LogUpdate "Now running with library version : $UPDATE_COMMON_SCRIPT_VERSION"

	return 0
}

#########################################################################################
#####								StartUpdate
#####
##### The process is divided in 3 steps : FIRMWARE - ENVIRONMENT - SOFTWARE
##### 
##### First of all we must consider the presence in the zipfile of a new "update_common" 
##### file which act as a library of functions that may be used by all scripts. 
##### Neither this script nor all that will be actualized by a new environment can't work 
##### without it. Therefore, we must "source" it immediatly  before doing anything else
#####
##### Once the library has been actualized, the "real" work can start.
##### If there is a firmware to update, it must be done first as part of it will be 
##### overwritten by the environment update. We stiil keep a rule saying that a
##### firmware can only be upgrader never downgraded. But as the version verification is
##### just only based on firmware name it's much more a vain wish than a real constraint
#####
##### The ending of the update process depends on what have been updated.
#########################################################################################

StartUpdate() {

	##### First make adjustments according to current hardware
	if ! Update_Init; then
		LogUpdate "ERROR ----- Can't determine hardware platform. Abort !"
		exit
	fi

	##### Creating update directory if needed as we'll write an execution status at this location

	if [ ! -d "$UPDATE_DIR" ] ; then
		LogUpdate "Creating $UPDATE_DIR directory"
		CreateDirectory $UPDATE_DIR 2>&1 >> $APP_LOGFILE
	fi

	##########################################################################################################
	##### We start by looking at a firmware version that we have to update before updating the environment
	##### otherwise changes made to environment will be overwritten.

	LogUpdate ">>>>>>>>>> FIRMWARE - FIRMWARE - FIRMWARE <<<<<<<<<<"

	# look at a $UPDATE_FIRM_DIR directory which may include a new firmware to install

	if [ -d "$UPDATE_FIRM_DIR" ]; then 																									# Firmware must be in a dedicated directory
		
		LogUpdate "$UPDATE_FIRM_DIR included"
		Update_Firmware
	else
		LogUpdate "$UPDATE_FIRM_DIR not included  - Firmware not changed"
	fi

	##########################################################################################################
	##### Then look at the presence of a new application to install
	##### In the link of the different parts that are managed by the update process, the software update MUST 
	##### BE achieved before the environment update as it may restart the "update daemon" if it has to be
	##### updated. If not, the freshly restarted "update daemon" will misconceived the software stop which 
	##### produce actually exactly the same effect as the software start

	LogUpdate ">>>>>>>>>> SOFTWARE - SOFTWARE - SOFTWARE <<<<<<<<<<"

	if [ -d $UPDATE_APP_DIR ]; then
		
		##### First verify that there are some files to copy under the $UPDATE_APP_DIR
		directory_content=$(ls -a $UPDATE_APP_DIR | sed -e "/\.$/d" | wc -l)
		
		if [ $directory_content -ne 0 ]; then
	
			# Ok some files are present, update can start
			LogUpdate "Files found in $UPDATE_APP_DIR. Starting upgrading process"

			SetLed POWER GREEN NORMAL_BLINK

			if ! Update_Software; then

				# Set leds to inform that update process ran into problems
				SetLed POWER ORANGE NORMAL_BLINK
				sleep 2
				return
			fi

			# On successfull update, the soft must be restarted and the survey daemon must control the correct execution of the new version
			must_restart_software=true
			SetUpdateStatus $STATUS_UPDATE_OK

		else
	
			##### An empty directory must not be considered as an error. It just stop the update_software process
			LogUpdate "No files present in $UPDATE_APP_DIR. Software not changed"
	
			# Current software doesn't have to be restarted and it must be considered like an happy end for the update daemon
			must_restart_software=false
			SetUpdateStatus $STATUS_APP_STARTED
		fi

	else
		LogUpdate "$UPDATE_APP_DIR not included - Software not changed"

		# Current software doesn't have to be restarted
		# and it must be considered like an happy end for the update daemon

		must_restart_software=false
		SetUpdateStatus $STATUS_APP_STARTED
	fi

	##########################################################################################################
	##### Then last, the environment can be updated.
	##### This is done lastly because of a potential update of the "update daemon" as it it explained above

	LogUpdate ">>>>>>>>>> ENVIRONMENT - ENVIRONMENT - ENVIRONMENT <<<<<<<<<<"

	# Look at a $UPDATE_ENV_DIR directory and a $MODENV_SCRIPT to execute

	if [ -d "$UPDATE_ENV_DIR" ]; then

		LogUpdate "$UPDATE_ENV_DIR included"
		LogUpdate "Looking at presence of $UPDATE_ENV_DIR/$MODENV_SCRIPT from $(pwd)"
		
		if [ -f "$UPDATE_ENV_DIR/$MODENV_SCRIPT" ]; then

			LogUpdate "$UPDATE_ENV_DIR/$MODENV_SCRIPT found. Set executable permissions"
			chmod 755 "$UPDATE_ENV_DIR/$MODENV_SCRIPT" 2>&1 >> $APP_LOGFILE

			cd $UPDATE_ENV_DIR
			
			SetLed POWER GREEN HEARTBEAT

			LogUpdate "Executing ./$MODENV_SCRIPT $FUNC_INSTALL from $(pwd) directory"
			./$MODENV_SCRIPT $FUNC_INSTALL

			cd ..
		else
			LogUpdate "$UPDATE_ENV_DIR/$MODENV_SCRIPT not found - Environment not changed"
		fi
	else
		LogUpdate "$UPDATE_ENV_DIR not included - Environment not changed"
	fi

	##########################################################################################################
	#------------------ Finally start application newly installed by launching application script if found.

	if [ "$must_restart_software" = true ]; then

		if [ ! -f "$USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT" ] ; then
			LogUpdate "$USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT not found ... Abort"

			SetLed POWER ORANGE NORMAL_BLINK
			sleep 1

		else
			
			SetLed POWER GREEN SLOW_BLINK

			LogUpdate "Launch $USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT update"
			$USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT update 
		fi

	else
			SetLed POWER GREEN FIXED
	fi

	LogUpdate "Update ended"
}

#########################################################################################
# Function that restore a previously backup version (software + environment)
#
# Starts to find a running version that must be stopped before being deinstalled.
# Then restore backuped environment if it exists
# Then restore old version from BACKUP_APP_DIR directory and start it
#
# Input : none
# Output : 0 = true / 1 = false
#########################################################################################

StartRestore() {

	# Remove application status file as it may had just been created by the application 

	if [ -f $APP_STATUS_FILE ]; then
		LogUpdate "Remove application status file : $APP_STATUS_FILE"
		rm "$APP_STATUS_FILE" 2>&1 >> $APP_LOGFILE
	else
		LogUpdate "No application status file : $APP_STATUS_FILE to remove"
	fi

	##### Look at process running and stop it if so
	
	if Is_Framework_Running; then

		LogUpdate "Framework is currently running. Beginning stopping process"

		# Then stop the framework
		if ! Stop_Software; then
			return 1
		fi

	fi
	
	##### Restore backuped environment if exists

	cd $UNZIP_DIR																										# Goes to working directory with all unzipped files
				
	if [ -d "$MODENV_DIR" ]; then

		if [ -f "$MODENV_DIR/$MODENV_SCRIPT" ]; then

			currentDir=$(pwd)

			cd $MODENV_DIR
			./$MODENV_SCRIPT $FUNC_RESTORE

			cd $currentDir
		fi

	fi

	##### Remove useless backuped environment 
	LogUpdate "Remove backuped environment"
	CleanDirectory $BACKUP_ENV_DIR

	##### Remove actual (but bad) version
	LogUpdate "Remove new application"
	CleanDirectory $APP_DIR
	
	##### And restore backuped version
	LogUpdate "Restore $BACKUP_APP_DIR application version"
	mv $BACKUP_APP_DIR $APP_DIR 2>&1 >> $$APP_LOGFILE

	##### Finally restart application 

	if [ ! -f "$USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT" ]; then
		LogUpdate "Application $USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT not found ... Abort"
	else
		SetLed POWER GREEN NORMAL_BLINK
		$USR_LOCAL_BIN/$HAGER_APP_STARTING_SCRIPT update 
	fi

	LogUpdate "Restore ended"

	return 0
}

#########################################################################################
StartClean() {

	# Delete application backup
	CleanDirectory $BACKUP_APP_DIR

	# Delete application data's backup if it exists
	if [ -e "$ROOT_DIR/$APP_BACKUP_FILE" ]; then
		LogUpdate "Delete $ROOT_DIR/$APP_BACKUP_FILE"
		rm "$ROOT_DIR/$APP_BACKUP_FILE" 2>&1 >> $$APP_LOGFILE
	fi

	# launch modify_environment script with same parameter so that it can clean its own things
	if [ -d "$MODENV_DIR" ]; then
		if [ -f "$MODENV_DIR/$MODENV_SCRIPT" ]; then

			currentDir=$(pwd)
			
			cd $MODENV_DIR
			./$MODENV_SCRIPT $FUNC_CLEAN
			
			cd $currentDir
		fi

	fi
			
	# Then environment backup can be deleted
	CleanDirectory $BACKUP_ENV_DIR

	return 0
}

#########################################################################################
#########################################################################################
#####								Process start
#####
##### First versions of update_survey didn't launch the update script with an argument
##### and took care of restoring what was backuped by it.
##### It was a too strong coupling between them....
#####
##### Now this script take care of more function but must still react when it is called 
##### without any argument as it did before, it means by installing the update.
#########################################################################################
#########################################################################################

Update_WriteLog "----------------------------- $(basename "$0") ($1) - v$UPDATE_SCRIPT_VERSION (using library $UPDATE_COMMON_SCRIPT_VERSION) - started"

##### Initialize all defines which depend on the location of the script which is de facto the update "rootfs"

UNZIP_DIR=$(pwd)
UPDATE_APP_DIR="$UNZIP_DIR/app"
UPDATE_FIRM_DIR="$UNZIP_DIR/firm"
UPDATE_ENV_DIR="$UNZIP_DIR/modify_environment"

##### Then, source the common library that must be included in update zip file and will be used by all functions

if ! Update_Source_New_Common_Lib; then
	Update_WriteLog "ERROR ----- Update aborted !!"
	InternalSetUpdateStatus	$INTERNAL_STATUS_UPDATE_ABORT
	exit 
fi

##### Then, look at the job to do

case "$1" in

	$FUNC_RESTORE)
		StartRestore
		;;

	$FUNC_CLEAN)
		StartClean
		;;

	$FUNC_INSTALL | *)
		StartUpdate
		;;

esac

LogUpdate "----------------------------- $(basename "$0") - v$UPDATE_SCRIPT_VERSION ended"
